// MainDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AXBoardMonitor.h"
#include "MainDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainDlg dialog


CMainDlg::CMainDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMainDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMainDlg)
	//}}AFX_DATA_INIT
}


void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMainDlg)
	DDX_Control(pDX, IDC_STATUS, m_status);
	//}}AFX_DATA_MAP

	DDX_Control(pDX, IDC_ADDISP, adchans);
	DDX_Array(IDC_ADCHAN0, adchanval, NUM_AD);
	DDX_Array(IDC_DACHAN0, dachans, NUM_DA);
	DDX_Array(IDC_DIOOUT0, dioout, NUM_DIO);
	DDX_Array(IDC_DIOIN0, dioin, NUM_DIO);
}


BEGIN_MESSAGE_MAP(CMainDlg, CDialog)
	//{{AFX_MSG_MAP(CMainDlg)
	ON_WM_COMPAREITEM()
	ON_WM_CTLCOLOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CMainDlg::HandleError(int rc, char* str)
{
	if (rc)
    {
		char buffer[1024];
        sprintf(buffer, "%s Error [%d]: %s", str, rc, dali.GetLastErrorString());
		m_status.SetWindowText(buffer); 		
    }

	return rc;
}


/////////////////////////////////////////////////////////////////////////////
// CMainDlg message handlers

int CMainDlg::OnCompareItem(int nIDCtl, LPCOMPAREITEMSTRUCT lpCompareItemStruct) 
{
	// TODO: Add your message handler code here and/or call default
	
	return CDialog::OnCompareItem(nIDCtl, lpCompareItemStruct);
}

BOOL CMainDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	
	char buffer[256];
	sprintf(buffer, "Connected to: http%s://%s:%d/dali/soap/%s (%s based)", useSSL ? "s" : "", remoteIP, remotePort, board, alarms ? "Alarm" : "Polling");
	m_status.SetWindowText(buffer);

	HandleError(dali.SetServer(remoteIP, remotePort, useSSL), "dali.SetServer()");
	HandleError(dali.GetBoardHandle(board, &bhandle), "dali.GetBoardHandle()");

	if (dali.Ping())
	{
		m_status.SetWindowText("Could not connect to server");
		return TRUE;
	}

	
	DALI_INTERFACE_SETTINGS dis;
	dis.dali_board = bhandle;
	
	//set parameters for the alarms if they are to be used
	dis.port = 5555; //random unused port?
	dis.alarm.duration = 1200; //20 minutes
	dis.alarm.period = 1;
	dis.alarm.postalarmpause = 1;

	//Register Interface for the A/D channels
	dis.ms_interval = 100;
	dis.low_channel_port = 0;
	dis.high_channel_bit = 15;
	dis.adsettings.gain = 1;
	dis.adsettings.polarity = DALI_BIPOLAR;
	dis.adsettings.range = 10;
	
	//register the static A/D channel readouts
	dis.operation_type = DALI_CMD_AD_SCAN_SPREAD;
	DALI_REGISTER_SPREAD(dali, dis, adchanval, NUM_AD, ihandle);

	//register the bar graph
	dis.operation_type = DALI_CMD_AD_SCAN;
	HandleError(dali.RegisterInterface(&dis, &adchans, &ihandle), "dali.RegisterInterface()");


	//Register D/A outputs, and get original D/A values
	for (int da = 0; da < NUM_DA; da++)
	{
		//register interface
		dis.operation_type = DALI_CMD_DA_CONVERT;
		dis.low_channel_port = da;
		HandleError(dali.RegisterInterface(&dis, &dachans[da], &ihandle), "dali.RegisterInterface()");
		
		dis.operation_type = DALI_CMD_DA_READBACK | (alarms ? DALI_CMD_ALARM : 0);
		HandleError(dali.RegisterInterface(&dis, &dachans[da], &ihandle), "dali.RegisterInterface()");

		//get D/A values
		int value;
		HandleError(dali.PerformDAReadBack(bhandle, da, &value), "dali.RegisterInterface()");		
		dachans[da].SetDispValue(value);
	}	

	

	//now register the DIO output port buttons, and get initial values
	for (int dio = 0; dio < NUM_DIO; dio++)
	{
		//register button output interfaces
		dis.operation_type = DALI_CMD_DIO_OUT_BIT;
		dis.low_channel_port = 0;
		dis.high_channel_bit = dio;
		HandleError(dali.RegisterInterface(&dis, &dioout[dio], &ihandle), "dali.RegisterInterface()");

		//register button input interfaces (assumes DIO port 0 is output)
		dis.operation_type = DALI_CMD_DIO_READBACK_BIT | (alarms ? DALI_CMD_ALARM : 0);
		HandleError(dali.RegisterInterface(&dis, &dioout[dio], &ihandle), "dali.RegisterInterface()");

		//register LED input interfaces (assumes DIO port 1 is input)
		dis.operation_type = DALI_CMD_DIO_IN_BIT | (alarms ? DALI_CMD_ALARM : 0);
		dis.low_channel_port = 1;
		HandleError(dali.RegisterInterface(&dis, &dioin[dio], &ihandle), "dali.RegisterInterface()");

		//get initial values
		int value;
		HandleError(dali.PerformDIOReadBackBit(bhandle, 0, dio, &value), "dali.PerformDIOReadBackBit()");
		dioout[dio].SetLEDState(value);
		HandleError(dali.PerformDIOInputBit(bhandle, 1, dio, &value), "dali.PerformDIOInputBit()");
		dioin[dio].SetDispValue(value);
	}




	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

HBRUSH CMainDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	brush.DeleteObject();
	brush.CreateSolidBrush(RGB(255,255,255));
	return (HBRUSH)brush.GetSafeHandle();
}
