// MainDlg.cpp : implementation file
//

#include "stdafx.h"
#include "AXDaliDemo.h"
#include "MainDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMainDlg dialog


CMainDlg::CMainDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMainDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMainDlg)
	//}}AFX_DATA_INIT	
}


void CMainDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMainDlg)
	DDX_Control(pDX, IDC_STATUS, m_status);
	//}}AFX_DATA_MAP

	DDX_Control(pDX, IDC_TEMP, m_temp);
	DDX_Array(IDC_LIGHT0, m_led, 4);
	DDX_Array(IDC_COLOR0, m_color, 3);
}


BEGIN_MESSAGE_MAP(CMainDlg, CDialog)
	//{{AFX_MSG_MAP(CMainDlg)
	ON_WM_CTLCOLOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

int CMainDlg::HandleError(int rc, char* str)
{
	if (rc)
    {
		char buffer[1024];
        sprintf(buffer, "%s Error [%d]: %s", str, rc, dali.GetLastErrorString());
		m_status.SetWindowText(buffer); 		
    }

	return rc;
}

/////////////////////////////////////////////////////////////////////////////
// CMainDlg message handlers

BOOL CMainDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
		
	char buffer[256];
	sprintf(buffer, "Connected to server");
	m_status.SetWindowText(buffer);

	HandleError(dali.SetServer(remoteIP, remotePort, useSSL), "dali.SetServer()");
	HandleError(dali.GetBoardHandle(board, &bhandle), "dali.GetBoardHandle()");

	if (dali.Ping())
	{		
		sprintf(buffer, "Could not connect to %s:%d", remoteIP, remotePort);
		m_status.SetWindowText(buffer);
		return TRUE;
	}

	
	DALI_INTERFACE_SETTINGS dis;
	dis.dali_board = bhandle;
	
	//set parameters for the alarms if they are to be used
	dis.port = 5555; //random unused port?
	dis.alarm.duration = 1200; //20 minutes
	dis.alarm.period = 1;
	dis.alarm.postalarmpause = 1;

	//Register Interface for the A/D channels
	dis.ms_interval = 100;
	dis.low_channel_port = 0;
	dis.high_channel_bit = 1;
	dis.adsettings.gain = 1;
	dis.adsettings.polarity = DALI_BIPOLAR;
	dis.adsettings.range = 10;
	
	//register the bar graph
	dis.operation_type = DALI_CMD_AD_SCAN;
	HandleError(dali.RegisterInterface(&dis, &m_temp, &ihandle), "dali.RegisterInterface()");


	//Register D/A outputs, and get original D/A values
	for (int da = 0; da < 3; da++)
	{
		//register interface
		dis.operation_type = DALI_CMD_DA_CONVERT;
		dis.low_channel_port = da;
		HandleError(dali.RegisterInterface(&dis, &m_color[da], &ihandle), "dali.RegisterInterface()");
		
		dis.operation_type = DALI_CMD_DA_READBACK | (alarms ? DALI_CMD_ALARM : 0);
		HandleError(dali.RegisterInterface(&dis, &m_color[da], &ihandle), "dali.RegisterInterface()");

		//get D/A values
		int value;
		HandleError(dali.PerformDAReadBack(bhandle, da, &value), "dali.RegisterInterface()");		
		m_color[da].SetDispValue(value);
	}	

	

	//now register the DIO output port buttons, and get initial values
	for (int dio = 0; dio < 4; dio++)
	{
		//register button output interfaces
		dis.operation_type = DALI_CMD_DIO_OUT_BIT;
		dis.low_channel_port = 0;
		dis.high_channel_bit = dio;
		HandleError(dali.RegisterInterface(&dis, &m_led[dio], &ihandle), "dali.RegisterInterface()");

		//register button input interfaces (assumes DIO port 0 is output)
		dis.operation_type = DALI_CMD_DIO_READBACK_BIT | (alarms ? DALI_CMD_ALARM : 0);
		HandleError(dali.RegisterInterface(&dis, &m_led[dio], &ihandle), "dali.RegisterInterface()");
	
		//get initial values
		int value;
		HandleError(dali.PerformDIOReadBackBit(bhandle, 0, dio, &value), "dali.PerformDIOReadBackBit()");
		m_led[dio].SetLEDState(value);		
	}
	
	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

HBRUSH CMainDlg::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	brush.DeleteObject();
	brush.CreateSolidBrush(RGB(255,255,255));
	return (HBRUSH)brush.GetSafeHandle();
}

