#include <stdio.h>
#include <dali_w32.h>


//These are the default values

//DALI_IP is the IP of the remote DALI server.  
#define DALI_IP   "127.0.0.1"

//DALI_PORT is the port of the remote DALI server.
#define DALI_PORT 8444

//DALI_DEMO is the name of the remote board to access.
#define DALI_DEMO "DEMO"


char* dali_ip;
int   dali_port;
char* dali_board;

int rc;
UINT demo;
CDali dali;


void HandleError(int rc, char* str)
{
	if (rc)
    {
        printf("%s Error [%d]: %s\n", str, rc, dali.GetLastErrorString());
		getchar();
        exit(rc);
    }
}

int main(int argc, char** argv)
{
	//Get command line args.
	//Usage: ConsoleServerInfo <remote ip> <remote port> <board name>

	dali_ip =    argc > 1 ? argv[1] : DALI_IP;
	dali_port =  argc > 2 ? atoi(argv[2]) : DALI_PORT;
	dali_board = argc > 3 ? argv[3] : DALI_DEMO;
	


	//First you must initialize the CDali object with a server IP, port and secure flag.
	//Setting the secure argument to '1' will make the CDali object use SSL (https://)
    
	printf("Server: %s\n", dali_ip);
	printf("Port:   %d\n", dali_port);
	printf("Board:  %s\n", dali_board);

    HandleError(dali.SetServer(dali_ip, dali_port, 0), "dali.SetServer()");
	



	//Now we must get the handle for the "DEMO" board.  The name of this board can be
	//modified in the defines above - it *must* match the name of the board on the server
	//you wish to communicate with.

	//Please remember that GetBoardHandle does more than just return a handle.  It allocates
	//and initializes data in the CDali object related to the board, and must be called at
	//least once before any operations that require the returned board handle.
	
	HandleError(dali.GetBoardHandle(dali_board, &demo), "dali.GetBoardHandle()");




	//PerformDAConvert will perform a D/A conversion on the remote board associated with the
	//'demo' handle.
    
	HandleError(dali.PerformDAConvert(demo, 0, 4095), "dali.PerformDAConvert()");




	//PerformADSample will return an A/D sample from the remote board associated with the
	//'demo' handle.  It requires a DALIADSETTINGS struct and a pointer to an int.

    int sample;
    DALIADSETTINGS adsettings;
    adsettings.gain = 1;
    adsettings.polarity = 0;
    adsettings.range = 1;
    
	HandleError(dali.PerformADSample(demo, 0, &adsettings, &sample), "dali.PerformADSample()");

    printf("AD Sample: %d\n", sample);
    getchar();

    return 0;

}