#include <stdio.h>
#include <dali_w32.h>

//These are default values

//DALI_IP is the IP of the remote DALI server.  
#define DALI_IP   "127.0.0.1"

//DALI_PORT is the port of the remote DALI server.
#define DALI_PORT 8444

//DALI_DEMO is the name of the remote board to access.
#define DALI_DEMO "DEMO"


#define LOW_CHAN  0
#define HIGH_CHAN 3
#define SCAN_INTERVAL 250



char* dali_ip;
int   dali_port;
char* dali_board;

CDali dali;
UINT demo;
UINT interface_handle;

void HandleError(int rc, char* str)
{
	if (rc)
    {
        printf("%s Error [%d]: %s\n", str, rc, dali.GetLastErrorString());
		getchar();
        exit(rc);
    }
}

UINT alarm_callback(int code, DALI_DATA* data)
{
	int i;

	switch (code)
	{
	case DALI_CMD_REGISTERED:
		printf("Successfully registered with server\n");
		break;

	case DALI_CMD_AD_SCAN:
		printf("Received scan: ");
		for (i = 0; i < (HIGH_CHAN - LOW_CHAN + 1); i++)
			printf("%8d", data->values[i]);
		printf("\n");
		break;

	default:
		printf("Received unexpected code: %X; val: %d\n", code, data->values[0]);
	}

	return 0;
}


int main(int argc, char ** argv)
{
	//Get command line args.
	//Usage: ConsoleServerInfo <remote ip> <remote port> <board name>

	dali_ip =    argc > 1 ? argv[1] : DALI_IP;
	dali_port =  argc > 2 ? atoi(argv[2]) : DALI_PORT;
	dali_board = argc > 3 ? argv[3] : DALI_DEMO;


	//First you must initialize the CDali object with a server IP, port and secure flag.
	//Setting the secure argument to '1' will make the CDali object use SSL (https://)
    
	printf("Server: %s\n",   dali_ip);
	printf("Port:   %d\n",   dali_port);
	printf("Board:  %s\n\n", dali_board);
	
    HandleError(dali.SetServer(dali_ip, dali_port, 0), "dali.SetServer()");

	
	//Now we must get the handle for the "DEMO" board.  The name of this board can be
	//modified in the defines above - it *must* match the name of the board on the server
	//you wish to communicate with.

	//Please remember that GetBoardHandle does more than just return a handle.  It allocates
	//and initializes data in the CDali object related to the board, and must be called at
	//least once before any operations that require the returned board handle.
	
	HandleError(dali.GetBoardHandle(dali_board, &demo), "dali.GetBoardHandle()");



	//Now to fill out the DALI_INTERFACE_SETTINGS struct for the scan operation.

	DALI_INTERFACE_SETTINGS dis;

	dis.dali_board = demo;                      //use the 'demo' board handle
	dis.operation_type = DALI_CMD_AD_SCAN;      //we're going to scan the A/D
	dis.low_channel_port = LOW_CHAN;            //specify the low channel
	dis.high_channel_bit = HIGH_CHAN;           //and the high channel
	dis.ms_interval = SCAN_INTERVAL;            //ms interval between scans
	dis.adsettings.range = 10;                  //set the A/D to +/- 10V (range 10, gain 1, pol BI)
	dis.adsettings.gain = 1;                    
	dis.adsettings.polarity = DALI_BIPOLAR;     

	

	//Now we will register the interface and print out the handle to the alarm

	HandleError(dali.RegisterInterface(&dis, alarm_callback, &interface_handle), "dali.RegisterInterface()");

	printf("Scan interface is regsitered with handle: %d\n", interface_handle);

	getchar();	//keep scrolling scans until enter is hit

	
	//Now we will unregister the interface.  Scans should stop.
	
	HandleError(dali.UnRegisterInterface(interface_handle), "dali.UnRegisterInterface()");
	printf("The interface has been unregistered\n\n");
	
	getchar();

	return 0;
}