#include <stdio.h>
#include <dali_w32.h>


//These are default values

//DALI_IP is the IP of the remote DALI server.  
#define DALI_IP   "127.0.0.1"

//DALI_PORT is the port of the remote DALI server.
#define DALI_PORT 8444

//DALI_DEMO is the name of the remote board to access.
#define DALI_DEMO "DEMO"


char* dali_ip;
int   dali_port;
char* dali_board;

CDali dali;
UINT numboards;

void HandleError(int rc, char* str)
{
	if (rc)
    {
        printf("%s Error [%d]: %s\n", str, rc, dali.GetLastErrorString());
		getchar();
        exit(rc);
    }
}


int main(int argc, char ** argv)
{
	//Get command line args.
	//Usage: ConsoleServerInfo <remote ip> <remote port> <board name>

	dali_ip =    argc > 1 ? argv[1] : DALI_IP;
	dali_port =  argc > 2 ? atoi(argv[2]) : DALI_PORT;
	dali_board = argc > 3 ? argv[3] : DALI_DEMO;


	//First you must initialize the CDali object with a server IP, port and secure flag.
	//Setting the secure argument to '1' will make the CDali object use SSL (https://)
    
	printf("Server: %s\n", dali_ip);
	printf("Port:   %d\n\n", dali_port);
	
    HandleError(dali.SetServer(dali_ip, dali_port, 0), "dali.SetServer()");




	//Now we will get the number of boards on the server.
	
	printf("Connecting to server...\n\n");

	HandleError(dali.GetNumBoards(&numboards), "dali.GetNumBoards()");

	printf("Number of boards: %d\n\n", numboards);
	


	//Now get information for each board.

	for (UINT b = 0; b < numboards; b++)
	{
		DALIBOARD db;
		UINT totalchans = 0;
		UINT numchan[5] = { 0,0,0,0,0 };


		//Retrieve the board info and number of channels.

		HandleError(dali.GetBoardInfo(b, &db), "dali.GetBoardInfo()");
		HandleError(dali.GetNumChannels(b, &totalchans), "dali.GetNumChannels()");


		//Now get all channel information.

		for (UINT ch = 0; ch < totalchans; ch++)
		{
			DALICHANNEL dc;
			dali.GetChannelInfo(b, ch, &dc);
			numchan[dc.type]++;
		}

		//Display board info.

		printf("Board %d\n", b);
		printf("   Board name:     %-12s Total channels:   %d\n", db.board, totalchans);
		printf("   Board type:     %-12d A/D channels:     %d\n", db.boardtype, numchan[1]);
		printf("   Base address:   %-12X D/A channels:     %d\n", db.baseaddress, numchan[2]);
		printf("                                DIO-in channels:  %d\n", numchan[3]);
		printf("                                DIO-out channels: %d\n", numchan[4]);

		printf("\n");
	}



	getchar();
	return 0;
}