/*
 * Copyright (c) 2003 by Diamond Systems Corporation. All Rights Reserved.
 * Use of this source code is subject to the terms of Diamond Systems 
 * Software License Agreement.
 *
 * DALI C/C++ Client demo program
 * See the DALI user manual for information on this API
 *
 * DALI client software must be installed on your system or this program
 * will not compile.  By default it will connect to a DALI server running
 * on your system.  Edit the env.service parameter below to talk to a
 * DALI server on a remote system.
 */

#include <stdio.h>
#include <stdlib.h>
#include "dalic.h"

int main(int argc, char *argv[])
{
	DALIENV env; // DALI client handle
	DALIADSETTINGS adsettings; // AD range/gain/polarity settings
	int channel; // Channel to sample
	int count; // Number of samples to take
	float hz; // Rate in samples per second
	int *samples; // Holds the data
	int result; // Holds the error code
	int i;

	/* Initialize the client.  Always call this first */
	DALIInit(&env);

	/* Service URL for the DALI server.  Switch to "https" for SSL
	 * encryption.  Change IP to talk to a different server */
	env.service = "http://127.0.0.1:8444/dali/soap/demo";

	channel = 0; // Samples from channel 0
	count = 10; // Take 10 samples total
	hz = 10; // At a rate of 10 samples per second

	adsettings.range = 5; // 5V operation
	adsettings.gain = 1; // AD gain 1
	adsettings.polarity = DALI_BIPOLAR;

	samples = malloc(sizeof(int) * count);
	if ( samples == NULL )
		return 1;

	/* This call will block for 1 second as 10 samples are taken at 10hz
	 * on the server.  Results are returned in one chunk */
	result = DALIADSampleMulti(&env, channel, count, hz, &adsettings, samples);
	if ( result )
		printf("DALI error: %s (%d)\n", env.last_error, result);
	else
		for ( i = 0; i < count; i++ )
			printf("DALI sample #%2d : %6d\n", i, samples[i]);

	free(samples);

	/* Closes any network connections and frees memory associated
	 * with the client handle */
	DALIFree(&env);
	return 0;
}

