/*
 * Copyright (c) 2003 by Diamond Systems Corporation. All Rights Reserved.
 * Use of this source code is subject to the terms of Diamond Systems 
 * Software License Agreement.
 *
 * DALI C/C++ Client demo program
 * See the DALI user manual for information on this API
 *
 * DALI client software must be installed on your system or this program
 * will not compile.  By default it will connect to a DALI server running
 * on your system.  Edit the env.service parameter below to talk to a
 * DALI server on a remote system.
 */

#include <stdio.h>
#include <string.h>
#include "dalic.h"

#define BUFSIZE 128

int main(int argc, char *argv[])
{
	DALIENV env; // DALI client handle
	DALIALARMSETTINGS alarmsettings; // Alarm configuration
	int channel; // Channel to monitor
	int bit; // Bit to select
	int result; // Holds the error code
	char alarmid[BUFSIZE];

	/* Initialize the client.  Always call this first */
	DALIInit(&env);

	/* Service URL for the DALI server.  Switch to "https" for SSL
	 * encryption.  Change IP to talk to a different server */
	env.service = "http://127.0.0.1:8444/dali/soap/demo";

	channel = 0;
	bit = 0;

	alarmsettings.period = 10; // Check once every 10 seconds
	alarmsettings.duration = 300; // Stop watching after 5 minutes
	/* Wait one minute before notifying again */
	alarmsettings.postalarmpause = 60;
	strcpy(alarmsettings.actiontype, "LOG"); // Write to logfile for alarm
	strcpy(alarmsettings.actiontarget, ""); // Not required

	result = DALIAlarmDIOOutOnChange(&env, channel, bit, &alarmsettings, 
					alarmid, BUFSIZE);
	if ( result )
		printf("DALI error: %s (%d)\n", env.last_error, result);
	else
		printf("DALI: Alarm ID: %s\n", alarmid);

	/* Closes any network connections and frees memory associated
	 * with the client handle */
	DALIFree(&env);
	return 0;
}

