// MFCBoardMonitorDlg.cpp : implementation file
//

#include "stdafx.h"
#include "MFCBoardMonitor.h"
#include "MFCBoardMonitorDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CMFCBoardMonitorDlg dialog

CMFCBoardMonitorDlg::CMFCBoardMonitorDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMFCBoardMonitorDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMFCBoardMonitorDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CMFCBoardMonitorDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMFCBoardMonitorDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP

	DDX_Array(IDC_ADCHAN0, adchans, NUM_AD);
	DDX_Array(IDC_ADVAL0, adchanval, NUM_AD);
	DDX_Array(IDC_DACHAN0, dachans, NUM_DA);
	DDX_Array(IDC_DIOABIT0, dioout, NUM_DIO);
	DDX_Array(IDC_DIOBBIT0, dioin, NUM_DIO);

}

BEGIN_MESSAGE_MAP(CMFCBoardMonitorDlg, CDialog)
	//{{AFX_MSG_MAP(CMFCBoardMonitorDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


void CMFCBoardMonitorDlg::HandleError(CDali* dali, int rc, char* str)
{
	if (rc)
    {
		char buffer[1024];
        sprintf(buffer, "%s Error [%d]: %s\n", str, rc, dali->GetLastErrorString());
		MessageBox(buffer, "DALI Error"); 
		exit(rc);
    }
}

/////////////////////////////////////////////////////////////////////////////
// CMFCBoardMonitorDlg message handlers

BOOL CMFCBoardMonitorDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon
	
	// TODO: Add extra initialization here

	for (int ad = 0; ad < NUM_AD; ad++)
	{
		adchans[ad].SetRange32(-32767, 32767);
	}

	for (int da = 0; da < NUM_DA; da++)
	{
		dachans[da].SetRange(0, 4095);
		dachans[da].SetTicFreq(409);
	}
	
	CServerSettingsDlg ssd;
	int rc = ssd.DoModal();
	if (rc == IDCANCEL)
	{
		exit(rc);
	}

	HandleError(&dali, dali.SetServer(ssd.m_address, ssd.m_port, ssd.m_useSSL), "dali.SetServer()");
	HandleError(&dali, dali.GetBoardHandle(ssd.m_boardname, &bhandle), "dali.GetBoardHandle()");
	
	if (dali.Ping())
	{
		MessageBox("Could not connect to server", "DALI Server Error");		
		exit(1);
	}

	DALI_INTERFACE_SETTINGS dis;
	dis.dali_board = bhandle;
		
	//Register Interface for the A/D channels
	dis.ms_interval = 100;
	dis.low_channel_port = 0;
	dis.high_channel_bit = 15;
	dis.adsettings.gain = 1;
	dis.adsettings.polarity = DALI_BIPOLAR;
	dis.adsettings.range = 10;
	dis.operation_type = DALI_CMD_AD_SCAN_SPREAD;
	
	DALI_REGISTER_SPREAD(dali, dis, adchans, NUM_AD, ihandle);
	DALI_REGISTER_SPREAD(dali, dis, adchanval, NUM_AD, ihandle);

	//set parameters for the alarms
	dis.port = 5555;
	dis.alarm.duration = 3600;
	dis.alarm.period = 1;
	dis.alarm.postalarmpause = 1;

	//Register D/A outputs, get original D/A values
	for (da = 0; da < NUM_DA; da++)
	{
		//register interface
		dis.operation_type = DALI_CMD_DA_CONVERT;
		dis.low_channel_port = da;
		HandleError(&dali, dali.RegisterInterface(&dis, &dachans[da], &ihandle), "dali.RegisterInterface()");
		
		dis.operation_type = DALI_CMD_DA_ALARM;
		HandleError(&dali, dali.RegisterInterface(&dis, &dachans[da], &ihandle), "dali.RegisterInterface()");

		//get D/A values
		int value;
		HandleError(&dali, dali.PerformDAReadBack(bhandle, da, &value), "dali.RegisterInterface()");
		dachans[da].SetPos(value);
	}	

	//now register the DIO output port buttons, and get initial values
	for (int dio = 0; dio < NUM_DIO; dio++)
	{
		dis.operation_type = DALI_CMD_DIO_OUT_BIT;
		dis.low_channel_port = 0;
		dis.high_channel_bit = dio;
		HandleError(&dali, dali.RegisterInterface(&dis, &dioout[dio], &ihandle), "dali.RegisterInterface()");

		dis.operation_type = DALI_CMD_DIO_RB_ALARM;
		HandleError(&dali, dali.RegisterInterface(&dis, &dioout[dio], &ihandle), "dali.RegisterInterface()");

		dis.operation_type = DALI_CMD_DIO_IN_ALARM;
		dis.low_channel_port = 0;
		HandleError(&dali, dali.RegisterInterface(&dis, &dioin[dio], &ihandle), "dali.RegisterInterface()");

		int value;
		HandleError(&dali, dali.PerformDIOReadBackBit(bhandle, 0, dio, &value), "dali.PerformDIOReadBackBit()");
		dioout[dio].SetCheck(value);
		HandleError(&dali, dali.PerformDIOInputBit(bhandle, 1, dio, &value), "dali.PerformDIOInputBit()");
		dioin[dio].SetCheck(value);
	}
	
	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CMFCBoardMonitorDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CMFCBoardMonitorDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

