/*

	dali_w32.h
	
	This is the main CDali class/constant/macro definition file.

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#ifndef __DALI_W32_H_
#define __DALI_W32_H_

#include <afxwin.h>
#include "dalic.h"

//Constants

#define DALI_MAX_CHANS     32
#define DALI_MAX_ERROR_LEN 128

#define DALI_MAX_ALARM_ID_LEN 128

//Errors
#define DALI_ERR_NONE              0x0000 /* No error */
#define DALI_ERR_COULDNOTCONNECT   0x0001 /* Could not connect to the server */
#define DALI_ERR_INVALID_PARAM     0x0002 /* Invalid parameter */
#define DALI_ERR_CONNECTIONFAIL    0x0003 /* Connection failed */
#define DALI_ERR_INVALID_BOARD     0x0004 /* Invalid board handle */
#define DALI_ERR_COULDNOTPERFORM   0x0005 /* Could not perform the operation */
#define DALI_ERR_NOFREEINTERFACE   0x0006 /* No free interface resources */
#define DALI_ERR_BOARDNOTFOUND     0x0007 /* Board handle not found */
#define DALI_ERR_TOOMANYBOARDS     0x0008 /* Too many board handles */
#define DALI_ERR_DECONSTRUCTING    0x0009 /* CDali object is currently deconstructing (for multi-threaded apps) */
 


//DALI MESSAGE ID

#define DALI_NOTIFY 0x7F00



//"WPARAM/code" DEFINES

#define DALI_CMD_ERROR             0x0000           /* Error */
#define DALI_CMD_REGISTERED        0x0001           /* Registering interface */
#define DALI_CMD_UNREGISTERED      0x4001           /* Unregistering interface */
#define DALI_CMD_AD_SAMPLE         0x0010           /* A/D sample interface */
#define DALI_CMD_AD_SCAN           0x0011           /* A/D scan interface */
#define DALI_CMD_DA_CONVERT        0x0220           /* D/A convert interface */
#define DALI_CMD_DA_READBACK       0x0021           /* D/A readback interface */
#define DALI_CMD_DIO_IN_BYTE       0x0030           /* DIO in-byte interface */
#define DALI_CMD_DIO_IN_BIT        0x0031           /* DIO in-bit interface */
#define DALI_CMD_DIO_READBACK_BYTE 0x0032           /* DIO readback-byte interface */
#define DALI_CMD_DIO_READBACK_BIT  0x0033           /* DIO readback-bit interface */
#define DALI_CMD_DIO_OUT_BYTE      0x0234           /* DIO out-byte interface */
#define DALI_CMD_DIO_OUT_BIT       0x0235           /* DIO out-bit interface */

#define DALI_CMD_OUTPUT            0x0200           /* If a code has this flag or'd in, it is an
                                                       'output' interface */

#define DALI_CMD_SPREAD                    0x0100   /* This flag or'd in indicates a spread interface */
#define DALI_CMD_AD_SCAN_SPREAD            0x0111   /* A/D scan spread interface */
#define DALI_CMD_DIO_IN_BYTE_SPREAD        0x0130   /* DIO in-byte spread interface */
#define DALI_CMD_DIO_READBACK_BYTE_SPREAD  0x0132   /* DIO readback-byte spread interface */

#define DALI_CMD_ALARM             0x0400           /* This flag or'd in indicates an alarm interface */
#define DALI_CMD_AD_ALARM          0x0410           /* A/D alarm interface */
#define DALI_CMD_DA_ALARM          0x0421           /* D/A alarm interface */
#define DALI_CMD_DIO_IN_ALARM      0x0431           /* DIO-in alarm interface */
#define DALI_CMD_DIO_RB_ALARM      0x0433           /* DIO-out alarm interface */


//"LPARAM/data" struct

typedef struct {

	int error;                            /* Specific error code - only valid when code = DALI_CMD_ERROR */
	char error_str[DALI_MAX_ERROR_LEN];   /* Error string associated with error */
	void* lpCDALI;                        /* Pointer to the CDali object that spawned this message */
	int interface_handle;                 /* Handle to the interface that spawned this message */
	int operation_type;                   /* Operation type of the parent interface (see above codes) */
	int dali_board;                       /* Handle to dali board in parent CDali object */

	int low_channel_port;                 /* Low channel/DIO port of operation (if valid) */
	int high_channel_bit;                 /* High channel/DIO bit of operation (if valid) */
	int values[DALI_MAX_CHANS];           /* Values array - array location validity depends on operation */

} DALI_DATA;




//Callback function define

typedef UINT (*DALI_CALLBACKFUNC) (int code, DALI_DATA* data); /* callback function for interface operations */


//DDX_Array is used in the DoDataExchange method of a MFC class.  It is used to register 
//an array of controls in preparation for a spread interface.  Examples of this can be found in
//DALI sample projects.
#define DDX_Array(idc, arrayname, arraysize) { for (int forvar = 0; forvar < arraysize; forvar++) DDX_Control(pDX, idc + forvar, arrayname[forvar]); }

//This macro is used to register a spread interface.  For example, this allows a single scan to send
//individual channel messages to X number of different controls.
#define DALI_REGISTER_SPREAD(daliclass, dis, name, size, handle) { CWnd* pa[DALI_MAX_CHANS]; for (int i = 0; i < size; i++) pa[i] = &(name[i]); daliclass.RegisterInterface(&dis, pa, &handle); }


#define DIS_OPER(dis, board, op_type, ms_int) { dis.dali_board = board; dis.operation_type = op_type; dis.ms_interval = ms_int; }
#define DIS_CHAN(dis, low_port, high_bit) { dis.low_channel_port = low_port; dis.high_channel_bit = high_bit; }
#define DIS_ADSET(dis, r, p, g) { dis.adsettings.gain = g; dis.adsettings.polarity = p; dis.adsettings.range = r; }
#define DIS_ALARM(dis, prt, dur, per, postpause) { dis.port = prt; dis.alarm.duration = dur; dis.alarm.period = per; dis.alarm.postalarmpause = postpause; }


//Constants
#define DALI_MAX_SERVER_SIZE 256
#define DALI_MAX_BOARDS      16
#define DALI_MAX_INTERFACES  100
#define DALI_MAX_SERVICE_LEN 256
#define DALI_MAX_USERID_LEN  32
#define DALI_MAX_PASSWD_LEN  32
#define DALI_MAX_IP_LEN      20


typedef struct {

	UINT dali_board;            /* Handle to the CDali board used for this interface */
	UINT operation_type;        /* Operation type for the interface (see codes above) */
	UINT low_channel_port;      /* Low channel/DIO port */
	UINT high_channel_bit;      /* High channel/DIO bit (if necessary) */
	UINT ms_interval;           /* Interval between data updates in ms (not used by alarms) */
	DALIADSETTINGS adsettings;  /* A/D settings for A/D sample or scans */
	
	//For alarms
	DALIALARMSETTINGS alarm;    /* Alarm data for alarm interfaces */
	int port;                   /* Local port which the server will connect back to (must be open) */
	int low_value;              /* Low value for A/D alarm range */
	int high_value;             /* High value for A/D alarm range */

} DALI_INTERFACE_SETTINGS;


//This is an internal struct
struct DALI_INTERFACE {
	
	DALI_INTERFACE_SETTINGS settings;

	UINT   spread;
	CWnd*  user_cwnd[32];
	DALI_CALLBACKFUNC user_func;
	LPTSTR service;
	BOOL   active;
	BOOL   registered;
	UINT   handle;
	ULONG  next_trigger;
	char   alarm_handle[DALI_MAX_ALARM_ID_LEN];

	struct DALI_INTERFACE * next;
	
};


//This is an internal class
class CDaliPQ
{
public:

	CDaliPQ();
	~CDaliPQ();

	void Lock();
	void UnLock();

	void InsertInterface(DALI_INTERFACE* di);
	DALI_INTERFACE* GetFirst();
	void PushFirst();

protected:

	HANDLE mutex;
	DALI_INTERFACE* queue;

	void InsertInOrder(DALI_INTERFACE* di);

};



class CDali
{
public:
	
	//Constructor/Destructor
	CDali();
	CDali(LPCTSTR name, UINT port, LPCTSTR userid = "", LPCTSTR passwd = "", BOOL secure_http = FALSE);
	virtual ~CDali();
	
	UINT SetServer(LPCTSTR name, UINT port, BOOL secure_http);
	UINT GetServer(LPTSTR name,  UINT max_name, UINT* port);	
	UINT GetLocalIP(LPTSTR ip);

	UINT SetUserID(LPCTSTR userid);
	UINT GetUserID(LPTSTR userid, UINT max_len);

	UINT SetPassword(LPCTSTR passwd);
	UINT GetPassword(LPTSTR passwd, UINT max_len);

	LPCTSTR GetLastErrorString();

	UINT GetBoardHandle(LPCTSTR board_name, UINT* board);

	UINT GetNumBoards(UINT* numboards);
	UINT GetBoardInfo(UINT board, DALIBOARD* boardinfo);
	
	UINT GetNumChannels(UINT board, UINT* numchan);
	UINT GetChannelInfo(UINT board, UINT chan, DALICHANNEL* chaninfo);

	UINT RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, CWnd* user_cwnd, UINT* int_handle);
	UINT RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, CWnd** user_cwnd, UINT* int_handle);	
	UINT RegisterInterface(DALI_INTERFACE_SETTINGS* dali_int, DALI_CALLBACKFUNC user_func, UINT* int_handle);
	UINT UnRegisterInterface(UINT handle);

	UINT PerformADSample(UINT board, UINT channel, DALIADSETTINGS* adsettings, int* value);
	UINT PerformADScan(UINT board, UINT low_channel, UINT high_channel, DALIADSETTINGS* adsettings, int* values);

	UINT PerformADSampleMulti(UINT board, UINT channel, UINT count, float hz, DALIADSETTINGS* adsettings, int* value);
	UINT PerformADScanMulti(UINT board, UINT low_channel, UINT high_channel, UINT count, float hz, DALIADSETTINGS* adsettings, int* values);

	UINT PerformDAConvert(UINT board, UINT channel, int value);
	UINT PerformDAReadBack(UINT board, UINT channel, int* value);

	UINT PerformDIOOutputByte(UINT board, UINT port, int value);
	UINT PerformDIOOutputBit(UINT board, UINT port, UINT bit, int value);
	
	UINT PerformDIOInputByte(UINT board, UINT port, int* value);
	UINT PerformDIOInputBit(UINT board, UINT port, UINT bit, int* value);

	UINT PerformDIOReadBackByte(UINT board, UINT port, int* value);
	UINT PerformDIOReadBackBit(UINT board, UINT port, UINT bit, int* value);

	UINT Ping();

	void ProcessPQ();
	void AlertThread();
	
protected:

	DALIENV env;

	char server[DALI_MAX_SERVER_SIZE];
	char server_soap[DALI_MAX_SERVER_SIZE];
	UINT port;
	BOOL alive;

	char userid[DALI_MAX_USERID_LEN];
	char passwd[DALI_MAX_PASSWD_LEN];

	char last_error[DALI_MAX_ERROR_LEN];

	UINT      numboards;
	DALIBOARD boards[DALI_MAX_BOARDS];
	char      board_soaps[DALI_MAX_BOARDS][DALI_MAX_SERVER_SIZE];

	UINT         numchans[DALI_MAX_BOARDS];
	DALICHANNEL* chans[DALI_MAX_BOARDS];
	
	UINT           numifaces;
	DALI_INTERFACE ifaces[DALI_MAX_INTERFACES];
				
	void DeleteChannelInfo();
	UINT SetLastError(UINT code, LPCTSTR errorstr);

	UINT SetupNewInterface(DALI_INTERFACE_SETTINGS* dali_int, void* user_hook, DALI_INTERFACE** di);
	void ActivateInterface(DALI_INTERFACE* di);

	CDaliPQ dalipq;
	UINT run_pq;	
	HANDLE pq_handle;
	DWORD pq_thread_id;

	UINT  run_at;
	HANDLE at_handle;
	DWORD at_thread_id;

	void PerformInterface(DALI_INTERFACE* di);
	void SendData(DALI_INTERFACE* di, int code, DALI_DATA* data);

	char my_ip[DALI_MAX_IP_LEN];
	int  my_port;
	
	UINT UpdateServerInfo();

	int  conn_est;
	void SetConnEst(int c);	
};



#endif //#ifndef