/*

	dali_pq.cpp
	
	This is an implementation of a very simple linked list priority queue to sort the interface events
	in order of their next trigger.  It has an O(1) peek/pop and O(N) push.

	The Lock/Unlocks were taken out for now because of a weird Win98 bug.  These work under
	WinNT/2K/XP so you may put them back in for thread safety.

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#include "dali_w32.h"
#include "dalic.h"


CDaliPQ::CDaliPQ()
{
	mutex = CreateMutex(NULL, FALSE, NULL);
	queue = NULL;
}

CDaliPQ::~CDaliPQ()
{
	CloseHandle(mutex);
}

void CDaliPQ::Lock()
{
	WaitForSingleObject(mutex, INFINITE);
}

void CDaliPQ::UnLock()
{
	ReleaseMutex(mutex);
}


void CDaliPQ::InsertInterface(DALI_INTERFACE* di)
{
	//Lock();

	di->next_trigger = GetTickCount();

	di->next = queue;
	queue = di;

	//UnLock();
}

DALI_INTERFACE* CDaliPQ::GetFirst()
{
	return queue;
}

void CDaliPQ::PushFirst()
{
	if (queue == NULL) return;

	//Lock();

	DALI_INTERFACE* di = queue;
	queue = di->next;

	//UnLock();

	di->next = NULL;

	di->next_trigger = GetTickCount() + di->settings.ms_interval;

	//if (di->active) 
		InsertInOrder(di);
}

void CDaliPQ::InsertInOrder(DALI_INTERFACE* di)
{
	//Lock();

	if (queue == NULL || queue->next_trigger > di->next_trigger)
	{
		di->next = queue;
		queue = di;
	//	UnLock();
		return;
	}

	DALI_INTERFACE* before = queue;

	while (before->next != NULL)
	{
		if (before->next->next_trigger > di->next_trigger)
		{
			di->next = before->next;
			before->next = di;
			//UnLock();
			return;
		}

		before = before->next;
	}

	before->next = di;
	//UnLock();
}