/*

	DALIButtonCtrl.cpp
	
	This is the implementation file for CDALIButtonCtrl

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#include "dalictrl.h"
#include "dali_w32.h"


/////////////////////////////////////////////////////////////////////////////
// CDALIButtonCtrl

CDALIButtonCtrl::CDALIButtonCtrl()
{
	CButton();	
	
	dali = NULL;
	last_pos = 0xFFFF;

	last_error_code = 0;
	last_error_str[0] = 0;

	cmd = 0;
}

CDALIButtonCtrl::~CDALIButtonCtrl()
{
	
}

BEGIN_MESSAGE_MAP(CDALIButtonCtrl, CButton)
	ON_WM_ERASEBKGND()
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_CONTROL_REFLECT(BN_CLICKED, OnUI)
END_MESSAGE_MAP()

LRESULT CDALIButtonCtrl::OnDALI(WPARAM code, LPARAM data)
{
	DALI_DATA* d = (DALI_DATA*)data;

	switch (code)
	{
	case DALI_CMD_ERROR:
		last_error_code = d->error;
		strcpy(last_error_str, d->error_str);
		EnableWindow(FALSE);
		break;

	case DALI_CMD_REGISTERED:
		dali = (CDali*)d->lpCDALI;

		if (d->operation_type == DALI_CMD_DA_CONVERT ||
			d->operation_type == DALI_CMD_DIO_OUT_BYTE ||
			d->operation_type == DALI_CMD_DIO_OUT_BIT )
		{
			cmd = d->operation_type;
			chan = d->low_channel_port;
			bit = d->high_channel_bit;
			board = d->dali_board;
		}
		
		last_error_code = 0;
		last_error_str[0] = 0;
		last_pos = GetState() & 1;
		EnableWindow(TRUE);
		break;

	default:
		EnableWindow(TRUE);
		SetState( (int)Convert(d->values[0]) );
		break;		
	}

	return 1;
}

double CDALIButtonCtrl::Convert(double value, BOOL incoming)
{
	return value;
}

void CDALIButtonCtrl::OnUI()
{
	if (dali == NULL || last_error_code || cmd == 0) return;

	int cur_pos = last_pos ? 0 : 1;
		
	last_pos = cur_pos;

	int oval = (int)Convert(cur_pos, FALSE);
	int ret_code = DALI_ERR_NONE;

	switch (cmd)
	{
	case DALI_CMD_DA_CONVERT:
		ret_code = dali->PerformDAConvert(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BYTE:
		ret_code = dali->PerformDIOOutputByte(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BIT:
		ret_code = dali->PerformDIOOutputBit(board, chan, bit, oval);
		break;
	}

	if (ret_code)
	{
		last_error_code = ret_code;
		strcpy(last_error_str, dali->GetLastErrorString());
		EnableWindow(FALSE);
	}

}


LPTSTR CDALIButtonCtrl::GetLastDALIError(UINT* code)
{
	*code = last_error_code;
	return last_error_str;
}