/*

	DALICheckBoxCtrl.cpp
	
	This is the implementation file for CDALICheckBoxCtrl

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#include "dalictrl.h"
#include "dali_w32.h"


/////////////////////////////////////////////////////////////////////////////
// CDALICheckBoxCtrl

CDALICheckBoxCtrl::CDALICheckBoxCtrl()
{
	CButton();	
	
	dali = NULL;

	last_error_code = 0;
	last_error_str[0] = 0;

	cmd = 0;
}

CDALICheckBoxCtrl::~CDALICheckBoxCtrl()
{
	
}

BEGIN_MESSAGE_MAP(CDALICheckBoxCtrl, CButton)
	ON_WM_ERASEBKGND()
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_CONTROL_REFLECT(BN_CLICKED, OnUI)
END_MESSAGE_MAP()

LRESULT CDALICheckBoxCtrl::OnDALI(WPARAM code, LPARAM data)
{
	DALI_DATA* d = (DALI_DATA*)data;

	switch (code)
	{
	case DALI_CMD_ERROR:
		last_error_code = d->error;
		strcpy(last_error_str, d->error_str);
		EnableWindow(FALSE);
		break;

	case DALI_CMD_REGISTERED:
		dali = (CDali*)d->lpCDALI;
		
		if (d->operation_type == DALI_CMD_DA_CONVERT ||
			d->operation_type == DALI_CMD_DIO_OUT_BYTE ||
			d->operation_type == DALI_CMD_DIO_OUT_BIT )
		{
			cmd = d->operation_type;
			chan = d->low_channel_port;
			bit = d->high_channel_bit;
			board = d->dali_board;
		}
		
		last_error_code = 0;
		last_error_str[0] = 0;
		EnableWindow(TRUE);
		break;

	default:
		EnableWindow(TRUE);
		SetCheck( (int)Convert(d->values[0]) );
		break;		
	}

	return 1;
}

double CDALICheckBoxCtrl::Convert(double value, BOOL incoming)
{
	return value;
}

void CDALICheckBoxCtrl::OnUI()
{
	if (dali == NULL || last_error_code || cmd == 0) return;

	int cur_pos = GetCheck() & 1;
	
	int oval = (int)Convert(cur_pos, FALSE);
	int ret_code = DALI_ERR_NONE;

	switch (cmd)
	{
	case DALI_CMD_DA_CONVERT:
		ret_code = dali->PerformDAConvert(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BYTE:
		ret_code = dali->PerformDIOOutputByte(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BIT:
		ret_code = dali->PerformDIOOutputBit(board, chan, bit, oval);
		break;
	}

	if (ret_code)
	{
		last_error_code = ret_code;
		strcpy(last_error_str, dali->GetLastErrorString());
		EnableWindow(FALSE);
	}

}


LPTSTR CDALICheckBoxCtrl::GetLastDALIError(UINT* code)
{
	*code = last_error_code;
	return last_error_str;
}