/*

	DALISliderCtrl.cpp
	
	This is the implementation file for CDALISliderCtrl

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#include "dalictrl.h"
#include "dali_w32.h"

#define TIMER_ONUI 5305
#define TIMER_ONUI_RATE 3000

/////////////////////////////////////////////////////////////////////////////
// CDALISliderCtrl

CDALISliderCtrl::CDALISliderCtrl()
{
	CSliderCtrl();	
	
	dali = NULL;
	last_pos = 0xFFFF;

	last_error_code = 0;
	last_error_str[0] = 0;

	cmd = 0;
	ignore_data = 0;
}

CDALISliderCtrl::~CDALISliderCtrl()
{
	
}

BEGIN_MESSAGE_MAP(CDALISliderCtrl, CSliderCtrl)
	ON_WM_ERASEBKGND()
	ON_WM_TIMER()
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_MESSAGE(WM_KEYDOWN, OnK)
	ON_MESSAGE(WM_MOUSEMOVE, OnM)
END_MESSAGE_MAP()

LRESULT CDALISliderCtrl::OnDALI(WPARAM code, LPARAM data)
{
	DALI_DATA* d = (DALI_DATA*)data;

	switch (code)
	{
	case DALI_CMD_ERROR:
		last_error_code = d->error;
		strcpy(last_error_str, d->error_str);
		EnableWindow(FALSE);
		break;

	case DALI_CMD_REGISTERED:	
		dali = (CDali*)d->lpCDALI;
		
		if (d->operation_type & DALI_CMD_OUTPUT)
		{
			cmd = d->operation_type;
			chan = d->low_channel_port;
			bit = d->high_channel_bit;
			board = d->dali_board;
		}

		last_error_code = 0;
		last_error_str[0] = 0;
		last_pos = GetPos();
		EnableWindow(TRUE);
		break;

	default:
		if (ignore_data) break;
		EnableWindow(TRUE);
		SetPos( (int)Convert(d->values[0]) );
		break;		
	}

	return 1;
}

double CDALISliderCtrl::Convert(double value, BOOL incoming)
{
	return value;
}

void CDALISliderCtrl::OnUI()
{
	if (dali == NULL || last_error_code || cmd == 0) return;

	int cur_pos = GetPos();
	
	if (cur_pos == last_pos) 
		return;

	last_pos = cur_pos;

	int oval = (int)Convert(cur_pos, FALSE);
	int ret_code = DALI_ERR_NONE;

	ignore_data = 1;

	switch (cmd)
	{
	case DALI_CMD_DA_CONVERT:
		ret_code = dali->PerformDAConvert(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BYTE:
		ret_code = dali->PerformDIOOutputByte(board, chan, oval);
		break;

	case DALI_CMD_DIO_OUT_BIT:
		ret_code = dali->PerformDIOOutputBit(board, chan, bit, oval);
		break;
	}

	SetTimer(TIMER_ONUI, TIMER_ONUI_RATE, 0);

	if (ret_code)
	{
		last_error_code = ret_code;
		strcpy(last_error_str, dali->GetLastErrorString());
		EnableWindow(FALSE);
	}
}

void CDALISliderCtrl::OnM(WPARAM wParam, LPARAM lParam)
{
	CSliderCtrl::OnMouseMove(wParam, CPoint(LOWORD(lParam),HIWORD(lParam)));
	OnUI();	
}

void CDALISliderCtrl::OnK(WPARAM wParam, LPARAM lParam)
{
	CSliderCtrl::OnKeyDown(wParam, LOWORD(lParam), HIWORD(lParam));
	OnUI();
}


LPTSTR CDALISliderCtrl::GetLastDALIError(UINT* code)
{
	*code = last_error_code;
	return last_error_str;
}

void CDALISliderCtrl::OnTimer(UINT nIDEvent)
{
	if (nIDEvent == TIMER_ONUI)
	{
		KillTimer(TIMER_ONUI);
		ignore_data = 0;
	}
}