/*

	dalictrl.h
	
	This is the main class/constant/macro definition file for all of the MFC DALI overrides.

	(c) Diamond Systems Corporation 2003

	http://www.diamondsystems.com

	(c) Copyright 2003 Diamond Systems Corporation. Use of this source code
	is subject to the terms of Diamond Systems' Software License Agreement.
	Diamond Systems provides no warranty of proper performance if this
	source code is modified.

	
*/

#ifndef __DALICTRL_H_
#define __DALICTRL_H_

#include <afxwin.h>
#include <afxcmn.h>

#include "dali_w32.h"

#define DALICTRL_ON_OFF  0
#define DALICTRL_BOOLEAN 1
#define DALICTRL_INTEGER 2
#define DALICTRL_DOUBLE  3


// ----------------------------------------------------------
//
//                     CDALIProgressCtrl
//
// ----------------------------------------------------------


class CDALIProgressCtrl : public CProgressCtrl
{

public:
	CDALIProgressCtrl();
	virtual ~CDALIProgressCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  
	virtual BOOL OnEraseBkgnd(CDC* pDC);

	LPTSTR GetLastDALIError(UINT* code);

protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALIProgressCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALIProgressCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALIProgressCtrl, CProgressCtrl)
	//{{AFX_MSG_MAP(CDALIProgressCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
END_MESSAGE_MAP()
*/

// ----------------------------------------------------------
//
//                      CDALISliderCtrl
//
// ----------------------------------------------------------

class CDALISliderCtrl : public CSliderCtrl
{

public:
	CDALISliderCtrl();
	virtual ~CDALISliderCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  
	virtual void OnTimer(UINT nIDEvent);

	LPTSTR GetLastDALIError(UINT* code);
	
protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	int last_pos;

	int ignore_data;

	UINT cmd;
	UINT chan;
	UINT bit;
	UINT board;
	
	void OnUI();
	void OnM(WPARAM wParam, LPARAM lParam);
	void OnK(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALISliderCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALISliderCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALISliderCtrl, CSliderCtrl)
	//{{AFX_MSG_MAP(CDALISliderCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_MESSAGE(WM_KEYDOWN, OnK)
	ON_MESSAGE(WM_MOUSEMOVE, OnM)
END_MESSAGE_MAP()
*/

// ----------------------------------------------------------
//
//                      CDALIButtonCtrl
//
// ----------------------------------------------------------

class CDALIButtonCtrl : public CButton
{

public:
	CDALIButtonCtrl();
	virtual ~CDALIButtonCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  

	LPTSTR GetLastDALIError(UINT* code);
	
protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	int last_pos;

	UINT cmd;
	UINT chan;
	UINT bit;
	UINT board;
	
	void OnUI();
		
	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALIButtonCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALIButtonCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALIButtonCtrl, CButton)
	//{{AFX_MSG_MAP(CDALIButtonCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_CONTROL_REFLECT(BN_CLICKED, OnUI)
END_MESSAGE_MAP()
*/


// ----------------------------------------------------------
//
//                      CDALICheckBoxCtrl
//
// ----------------------------------------------------------

class CDALICheckBoxCtrl : public CButton
{

public:
	CDALICheckBoxCtrl();
	virtual ~CDALICheckBoxCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  

	LPTSTR GetLastDALIError(UINT* code);
	
protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	UINT cmd;
	UINT chan;
	UINT bit;
	UINT board;
	
	void OnUI();
		
	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALICheckBoxCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALICheckBoxCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALICheckBoxCtrl, CButton)
	//{{AFX_MSG_MAP(CDALICheckBoxCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
	ON_CONTROL_REFLECT(BN_CLICKED, OnUI)
END_MESSAGE_MAP()
*/

// ----------------------------------------------------------
//
//                     CDALIStaticCtrl
//
// ----------------------------------------------------------


class CDALIStaticCtrl : public CStatic
{

public:
	CDALIStaticCtrl();
	virtual ~CDALIStaticCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  
	virtual BOOL OnEraseBkgnd(CDC* pDC);

	LPTSTR GetLastDALIError(UINT* code);

	void SetDisplayType(int dispType);

protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	UINT dt;

	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALIStaticCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALIStaticCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALIStaticCtrl, CStatic)
	//{{AFX_MSG_MAP(CDALIStaticCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
END_MESSAGE_MAP()
*/


// ----------------------------------------------------------
//
//                     CDALIEditCtrl
//
// ----------------------------------------------------------


class CDALIEditCtrl : public CEdit
{

public:
	CDALIEditCtrl();
	virtual ~CDALIEditCtrl();

	virtual LRESULT OnDALI(WPARAM code, LPARAM data);
	virtual double Convert(double value, BOOL incoming = TRUE);  
	virtual BOOL OnEraseBkgnd(CDC* pDC);

	LPTSTR GetLastDALIError(UINT* code);

	void SetDisplayType(int dispType);

protected:	

	CDali* dali;

	UINT last_error_code;
	char last_error_str[DALI_MAX_ERROR_LEN];

	UINT dt;

	DECLARE_MESSAGE_MAP()
};

/*
//{{AFX_MSG(CDALIEditCtrl)
//}}AFX_MSG
//{{AFX_VIRTUAL(CDALIEditCtrl)
//}}AFX_VIRTUAL
BEGIN_MESSAGE_MAP(CDALIEditCtrl, CEdit)
	//{{AFX_MSG_MAP(CDALIEditCtrl)
	//}}AFX_MSG_MAP
	ON_MESSAGE(DALI_NOTIFY, OnDALI)
END_MESSAGE_MAP()
*/

	
	
	



#endif